package com.masterandroid.convertidordivisas;

import android.app.ProgressDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.graphics.Color;
import android.graphics.drawable.Drawable;
import android.os.AsyncTask;
import android.support.v7.app.AppCompatActivity;
import android.os.Bundle;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.view.Window;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.Spinner;
import android.widget.TextView;
import android.widget.Toast;

import org.xml.sax.Attributes;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.DefaultHandler;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.net.URL;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

public class MainActivity extends AppCompatActivity {

    private Spinner spinnerOrigin;
    private Spinner spinnerEnd;
    private EditText cantidad;
    private TextView resultado;

    public void setRatio(double ratio) {
        this.ratio = ratio;
        String cantidad = this.cantidad.getText().toString();
        if(!cantidad.equals(""))
        {
            double cantidadDouble = Double.parseDouble(cantidad);
            double result = cantidadDouble * ratio;

            resultado.setText(""+result +" "+ spinnerEnd.getSelectedItem().toString());
        }
    }

    private double ratio;
    private Button btnConvert;
    private static MainActivity instance;

    String[] ratesCodes = {};
    Integer[] flagsIds = {};

    public MainActivity() {
        instance = this;
    }
    public static Context getContext() {
        return instance;
    }
    public static int getDrawable(String name) {
        Context context = MainActivity.getContext();
        name = name.toLowerCase();
        int id = context.getResources().getIdentifier("drawable/" + name, null, context.getPackageName());
        return id;
    }

    @Override
    protected void onCreate(Bundle savedInstanceState) {

        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        ratesCodes = getResources().getStringArray(R.array.ratesArray);
        flagsIds = new Integer[ratesCodes.length];

        for(int x = 0;x<ratesCodes.length;x++)
           flagsIds[x] = getDrawable(ratesCodes[x]);

        addListenerOnButton();
        addListenerOnSpinnerItemSelection();

        spinnerOrigin.setAdapter(new MyAdapter(MainActivity.this, R.layout.celdaspinner, ratesCodes));
        spinnerEnd.setAdapter(new MyAdapter(MainActivity.this,R.layout.celdaspinner,ratesCodes));
    }


    public class MyAdapter extends ArrayAdapter {

        public MyAdapter(Context context, int textViewResourceId, String[] objects)
        {
            super(context, textViewResourceId, objects);
        }

        public View getCustomView(int position, View convertView, ViewGroup parent)
        {
            LayoutInflater inflater = getLayoutInflater();
            View layout = inflater.inflate(R.layout.celdaspinner, parent, false);

            TextView tvLanguage = (TextView) layout.findViewById(R.id.rateName);
            ImageView img = (ImageView) layout.findViewById(R.id.rateFlag);

            tvLanguage.setText(getString(getResources().getIdentifier(ratesCodes[position], "string", getPackageName())));
            tvLanguage.setTextColor(Color.BLACK);

            img.setImageResource(flagsIds[position]);

            if (position == 0)
            {
                img.setVisibility(View.GONE);
                tvLanguage.setText("Seleccione un tipo de moneda.");
                //tvLanguage.setTextSize(50f);
                tvLanguage.setTextColor(Color.BLACK);
            }

            return layout;
        }

        @Override
        public View getDropDownView(int position, View convertView, ViewGroup parent)
        {
            return getCustomView(position, convertView, parent);
        }
        @Override
        public View getView(int position, View convertView, ViewGroup parent)
        {
            return getCustomView(position, convertView, parent);
        }
    }


    public void addListenerOnSpinnerItemSelection() {
        spinnerOrigin = (Spinner) findViewById(R.id.spinnerOrigen);
        //spinnerOrigin.setOnItemSelectedListener((AdapterView.OnItemSelectedListener) new CustomOnItemSelectedListener());

        spinnerEnd = (Spinner) findViewById(R.id.spinnerFinal);
        //spinnerEnd.setOnItemSelectedListener((AdapterView.OnItemSelectedListener) new CustomOnItemSelectedListener());
    }

    public void addListenerOnButton() {

        spinnerOrigin = (Spinner) findViewById(R.id.spinnerOrigen);
        spinnerEnd = (Spinner) findViewById(R.id.spinnerFinal);
        btnConvert = (Button) findViewById(R.id.convertbutton);
        cantidad = (EditText) findViewById(R.id.cantidadText);
        resultado = (TextView) findViewById(R.id.cantidadfinalLabel);

        btnConvert.setOnClickListener(new View.OnClickListener() {

            @Override
            public void onClick(View v) {

                new CalcularRatio().execute(spinnerOrigin.getSelectedItem().toString(),spinnerEnd.getSelectedItem().toString());



            }

        });
    }






    protected String resultadosSW(String currencyOrigin, String currencyTo) throws Exception
    {
        URL url = new URL("https://www.google.com/finance/converter?a=1&from=" + currencyOrigin + "&to=" + currencyTo);

        BufferedReader in = new BufferedReader(new InputStreamReader(url.openStream()));
        String str;
        String result = "";
        while ((str = in.readLine()) != null)
        {
            result += str;
        }
        in.close();
        Pattern mPattern = Pattern.compile("<span class=bld>[\\s\\S]*?<");
        Matcher matcher = mPattern.matcher(result.toString());
        String superResult = "";
        if(matcher.find())
        {
            superResult = matcher.group(0);
            superResult = superResult.split(">")[1];
            superResult = superResult.split(" ")[0];
        }
        return superResult;
    }


    public class ManejadorXML extends DefaultHandler {
        private String totalResults;
        private StringBuilder cadena = new StringBuilder();

        public String getRatioCurrency() {
            return totalResults;
        }

        @Override
        public void startElement(String uri, String nombreLocal, String
                nombreCualif, Attributes atributos) throws SAXException {
            cadena.setLength(0);
        }

        @Override
        public void characters(char ch[], int comienzo, int lon) {
            cadena.append(ch, comienzo, lon);
        }

        @Override
        public void endElement(String uri, String nombreLocal,
                               String nombreCualif) throws SAXException {
            totalResults = cadena.toString();
        }
    }

    class CalcularRatio extends AsyncTask<String, Void, String> {
        private ProgressDialog progreso;

        @Override
        protected void onPreExecute() {
            progreso = new ProgressDialog(MainActivity.this);
            progreso.setProgressStyle(ProgressDialog.STYLE_SPINNER);
            progreso.setMessage("Actualizando valor de divisas...");
            progreso.setCancelable(false);
            progreso.show();
        }

        @Override
        protected String doInBackground(String... palabras) {
            try {
                return resultadosSW(palabras[0], palabras[1]);
            } catch (Exception e) {
                cancel(true);
                Log.e("HTTP", e.getMessage(), e);
                return null;
            }
        }

        @Override
        protected void onPostExecute(String res) {
            progreso.dismiss();
            if (res!=null && !res.equals("")) {
                try {
                    ratio = Double.parseDouble(res);
                } catch (Exception ratioException) {
                    ratio = 1.0;
                }
                setRatio(ratio);
            }
        }

        @Override
        protected void onCancelled() {
            progreso.dismiss();
            ratio = 1.0;
            setRatio(ratio);
        }
    }


}
